package pt.up.dcc.vcandroid;

import java.util.ArrayList;
import java.util.HashMap;

import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.hardware.Camera;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.Window;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.Toast;

public class MainActivity extends Activity {
	private String vTAG = "MainActivity";
	private Camera mCamera;
	private CameraPreview mPreview;
	private ArrayList<String> cameraSizes;
	private HashMap<String, Camera.Size> supportedCamSize;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
		setContentView(R.layout.camera_layout);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// cria o menu com as opcoes para mudar o tamanho da frame 
		for (String key : cameraSizes) {
			menu.add(key);
		}
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		if (supportedCamSize.containsKey(item.getTitle())) {
			Camera.Size s = supportedCamSize.get(item.getTitle());
			mPreview.setCameraSize(s);
		}
		return true;
	}

	@Override
	protected void onPause() {
		releaseCamera();
		super.onPause();
	}

	@Override
	protected void onResume() {
		super.onResume();
		// inicializa a  camera
		if (!initCamera(getApplicationContext())) {
			Toast.makeText(getApplicationContext(), "ERRO ao iniciar a camera", Toast.LENGTH_SHORT).show();
		}
	}

	//////////////////////////////////////////////////

	/** inicializa camera */
	private boolean initCamera(Context context) {
		// Verificar se o dispositivo tem uma camera
		if (checkCameraHardware(context)) {
			try {
				// Create an instance of Camera
				mCamera = getCameraInstance();

				// Create our Preview view and set it as the content of our activity.
				mPreview = new CameraPreview(this, mCamera);
				FrameLayout preview = (FrameLayout) findViewById(R.id.camera_preview);
				preview.addView(mPreview);

				// listar as resolucoes existentes 
				Camera.Parameters p = mCamera.getParameters();
				supportedCamSize = new HashMap<String, Camera.Size>();
				cameraSizes = new ArrayList<String>();
				for (Camera.Size s : p.getSupportedPreviewSizes()) {
					String key = "" + s.width + "x" + s.height;
					supportedCamSize.put(key, s);
					cameraSizes.add(key);
				}
				return true;
			} catch (Exception e) {
				Log.e(vTAG, "Error init camrera", e);
			}
		}
		return false;
	}

	/** Check if this device has a camera */
	private boolean checkCameraHardware(Context context) {
		if (context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA)) {
			// this device has a camera
			return true;
		} else {
			// no camera on this device
			return false;
		}
	}

	/** A safe way to get an instance of the Camera object. */
	public static Camera getCameraInstance() {
		Camera c = null;
		try {
			c = Camera.open(); // attempt to get a Camera instance

		} catch (Exception e) {
			// Camera is not available (in use or does not exist)
			Log.e("", "Camera is not available (in use or does not exist)", e);
		}
		return c; // returns null if camera is unavailable
	}

	private void releaseCamera() {
		if (mCamera != null) {
			mCamera.setPreviewCallback(null);
			mCamera.stopPreview();
			mCamera.release(); // release the camera for other applications
			mCamera = null;
		}
	}
}
